import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _createForOfIteratorHelper from "@babel/runtime/helpers/esm/createForOfIteratorHelper";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
function rotatePoints(points, center, degrees) {
  if (points && points.length) {
    var _center = _slicedToArray(center, 2),
      cx = _center[0],
      cy = _center[1];
    var angle = Math.PI / 180 * degrees;
    var cos = Math.cos(angle);
    var sin = Math.sin(angle);
    var _iterator = _createForOfIteratorHelper(points),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var p = _step.value;
        var _p = _slicedToArray(p, 2),
          x = _p[0],
          y = _p[1];
        p[0] = (x - cx) * cos - (y - cy) * sin + cx;
        p[1] = (x - cx) * sin + (y - cy) * cos + cy;
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }
}
function rotateLines(lines, center, degrees) {
  var points = [];
  lines.forEach(function (line) {
    return points.push.apply(points, _toConsumableArray(line));
  });
  rotatePoints(points, center, degrees);
}
function areSamePoints(p1, p2) {
  return p1[0] === p2[0] && p1[1] === p2[1];
}
export function hachureLines(polygons, hachureGap, hachureAngle) {
  var hachureStepOffset = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;
  var angle = hachureAngle;
  var gap = Math.max(hachureGap, 0.1);
  var polygonList = polygons[0] && polygons[0][0] && typeof polygons[0][0] === 'number' ? [polygons] : polygons;
  var rotationCenter = [0, 0];
  if (angle) {
    var _iterator2 = _createForOfIteratorHelper(polygonList),
      _step2;
    try {
      for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
        var polygon = _step2.value;
        rotatePoints(polygon, rotationCenter, angle);
      }
    } catch (err) {
      _iterator2.e(err);
    } finally {
      _iterator2.f();
    }
  }
  var lines = straightHachureLines(polygonList, gap, hachureStepOffset);
  if (angle) {
    var _iterator3 = _createForOfIteratorHelper(polygonList),
      _step3;
    try {
      for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
        var _polygon = _step3.value;
        rotatePoints(_polygon, rotationCenter, -angle);
      }
    } catch (err) {
      _iterator3.e(err);
    } finally {
      _iterator3.f();
    }
    rotateLines(lines, rotationCenter, -angle);
  }
  return lines;
}
function straightHachureLines(polygons, gap, hachureStepOffset) {
  var vertexArray = [];
  var _iterator4 = _createForOfIteratorHelper(polygons),
    _step4;
  try {
    for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
      var polygon = _step4.value;
      var _vertices = _toConsumableArray(polygon);
      if (!areSamePoints(_vertices[0], _vertices[_vertices.length - 1])) {
        _vertices.push([_vertices[0][0], _vertices[0][1]]);
      }
      if (_vertices.length > 2) {
        vertexArray.push(_vertices);
      }
    }
  } catch (err) {
    _iterator4.e(err);
  } finally {
    _iterator4.f();
  }
  var lines = [];
  // eslint-disable-next-line no-param-reassign
  gap = Math.max(gap, 0.1);
  // Create sorted edges table
  var edges = [];
  for (var _i = 0, _vertexArray = vertexArray; _i < _vertexArray.length; _i++) {
    var vertices = _vertexArray[_i];
    for (var i = 0; i < vertices.length - 1; i++) {
      var p1 = vertices[i];
      var p2 = vertices[i + 1];
      if (p1[1] !== p2[1]) {
        var ymin = Math.min(p1[1], p2[1]);
        edges.push({
          ymin: ymin,
          ymax: Math.max(p1[1], p2[1]),
          x: ymin === p1[1] ? p1[0] : p2[0],
          islope: (p2[0] - p1[0]) / (p2[1] - p1[1])
        });
      }
    }
  }
  edges.sort(function (e1, e2) {
    if (e1.ymin < e2.ymin) {
      return -1;
    }
    if (e1.ymin > e2.ymin) {
      return 1;
    }
    if (e1.x < e2.x) {
      return -1;
    }
    if (e1.x > e2.x) {
      return 1;
    }
    if (e1.ymax === e2.ymax) {
      return 0;
    }
    return (e1.ymax - e2.ymax) / Math.abs(e1.ymax - e2.ymax);
  });
  if (!edges.length) {
    return lines;
  }
  // Start scanning
  var activeEdges = [];
  var y = edges[0].ymin;
  var iteration = 0;
  while (activeEdges.length || edges.length) {
    if (edges.length) {
      var ix = -1;
      for (var _i2 = 0; _i2 < edges.length; _i2++) {
        if (edges[_i2].ymin > y) {
          break;
        }
        ix = _i2;
      }
      var removed = edges.splice(0, ix + 1);
      // eslint-disable-next-line @typescript-eslint/no-loop-func
      removed.forEach(function (edge) {
        activeEdges.push({
          s: y,
          edge: edge
        });
      });
    }
    // eslint-disable-next-line @typescript-eslint/no-loop-func
    activeEdges = activeEdges.filter(function (ae) {
      if (ae.edge.ymax <= y) {
        return false;
      }
      return true;
    });
    activeEdges.sort(function (ae1, ae2) {
      if (ae1.edge.x === ae2.edge.x) {
        return 0;
      }
      return (ae1.edge.x - ae2.edge.x) / Math.abs(ae1.edge.x - ae2.edge.x);
    });
    // fill between the edges
    if (hachureStepOffset !== 1 || iteration % gap === 0) {
      if (activeEdges.length > 1) {
        for (var _i3 = 0; _i3 < activeEdges.length; _i3 = _i3 + 2) {
          var nexti = _i3 + 1;
          if (nexti >= activeEdges.length) {
            break;
          }
          var ce = activeEdges[_i3].edge;
          var ne = activeEdges[nexti].edge;
          lines.push([[Math.round(ce.x), y], [Math.round(ne.x), y]]);
        }
      }
    }
    y += hachureStepOffset;
    activeEdges.forEach(function (ae) {
      ae.edge.x = ae.edge.x + hachureStepOffset * ae.edge.islope;
    });
    iteration++;
  }
  return lines;
}