"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard").default;
var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _react = require("react");
var zrender = _interopRequireWildcard(require("zrender"));
require("./index.less");
var _useGanttChartStore = require("./store/useGanttChartStore");
var _hachure = require("./utils/hachure");
var _holidays = require("./utils/holidays");
var _task = require("./utils/task");
var _jsxRuntime = require("react/jsx-runtime");
var TimeLine = function TimeLine(props) {
  var _props$tasks = props.tasks,
    tasks = _props$tasks === void 0 ? [] : _props$tasks;
  var _useSearchParamsActio = (0, _useGanttChartStore.useSearchParamsActions)(),
    refreshSearchParamsStore = _useSearchParamsActio.refreshSearchParamsStore;
  var _useSearchParams = (0, _useGanttChartStore.useSearchParams)(),
    initChartStartX = _useSearchParams.initChartStartX,
    initChartStartY = _useSearchParams.initChartStartY,
    timeScaleHeight = _useSearchParams.timeScaleHeight,
    milestoneTopHeight = _useSearchParams.milestoneTopHeight,
    unitWidth = _useSearchParams.unitWidth,
    barHeight = _useSearchParams.barHeight,
    barMargin = _useSearchParams.barMargin,
    halfUnitWidth = _useSearchParams.halfUnitWidth,
    taskNamePaddingLeft = _useSearchParams.taskNamePaddingLeft;
  (0, _react.useEffect)(function () {
    refreshSearchParamsStore();
  }, [location]);
  var handleInitTimeLine = function handleInitTimeLine() {
    var container = document.getElementById('TimeLine');
    var zr = zrender.init(container);
    // margin left to the container
    var chartStartX = initChartStartX;
    // margin top to the container
    var chartStartY = Math.max(initChartStartY, timeScaleHeight + milestoneTopHeight);

    // 1. 拿到画布的宽
    var canvasWidth = zr.getWidth();
    var canvasHeight = zr.getHeight();
    // 2. 计算需要画多少格
    var timeScaleWidth = Math.ceil(canvasWidth / unitWidth);

    // 3. 画时间轴的矩形，设置位置x,y,长宽，给一个背景色填充
    var timeScale = new zrender.Rect({
      shape: {
        x: chartStartX,
        y: chartStartY - timeScaleHeight,
        width: timeScaleWidth * unitWidth,
        height: timeScaleHeight
      },
      style: {
        fill: 'rgba(255, 0,0, .2)'
      }
    });
    zr.add(timeScale);
    var lastScrollX = 0;
    var gridStartX = chartStartX;
    var gridEndX = timeScaleWidth * unitWidth;
    var gridLineCount = timeScaleWidth + 1;
    var deltaScrollX = Math.floor(lastScrollX / unitWidth);

    // 3. 遍历要画的线的个数
    for (var i = 0 + deltaScrollX, count = 0; count < gridLineCount; i++, count++) {
      var gridX = gridStartX + i * unitWidth;
      // 4. 画一根线，从（x1, y1） -> (x2, y2)
      var gridLine = new zrender.Line({
        shape: {
          x1: gridX,
          y1: chartStartY - timeScaleHeight,
          x2: gridX,
          y2: chartStartY + (barHeight + barMargin) * tasks.length
        },
        style: {
          stroke: 'lightgray'
        }
      });
      // 1. 线比格子多1，所以要提前1步退出
      if (count < gridLineCount - 1) {
        // MARK: 同一个遍历画「休息日斜线」
        var now = +new Date('2024-01-01');
        var currentDate = now + i * 60 * 1000 * 60 * 24;
        var dateInfo = (0, _holidays.isHoliday)(currentDate);
        // 是休息日的话画斜线
        if (dateInfo.isHoliday) {
          try {
            // 返回要画的线的开始、结束坐标
            var lines = (0, _hachure.hachureLines)([[chartStartX + i * unitWidth, chartStartY], [chartStartX + i * unitWidth + unitWidth, chartStartY], [chartStartX + i * unitWidth + unitWidth, chartStartY + (barHeight + barMargin) * tasks.length], [chartStartX + i * unitWidth, chartStartY + (barHeight + barMargin) * tasks.length]], 10, 45);
            // 用zrender画线段，描边上色
            lines.forEach(function (line) {
              var _line$ = (0, _slicedToArray2.default)(line[0], 2),
                x1 = _line$[0],
                y1 = _line$[1];
              var _line$2 = (0, _slicedToArray2.default)(line[1], 2),
                x2 = _line$2[0],
                y2 = _line$2[1];
              var l = new zrender.Line({
                shape: {
                  x1: x1,
                  y1: y1,
                  x2: x2,
                  y2: y2
                },
                style: {
                  stroke: 'rgba(221, 221, 221, 0.7)'
                }
              });
              zr.add(l);
            });
          } catch (error) {
            console.log(error);
          }
        }

        // 2. 画基本文本，可以直接改成日期名字，这里直接写遍历的index
        var dateText = new zrender.Text({
          style: {
            // text: i,
            text: dateInfo.dateString,
            x: gridX,
            y: chartStartY - timeScaleHeight
          },
          z: 1
        });
        // 3. 为了居中，要算出文本的宽高
        var _dateText$getBounding = dateText.getBoundingRect(),
          width = _dateText$getBounding.width,
          height = _dateText$getBounding.height;
        // 4. 重新设置日期文本位置，居中
        dateText.attr({
          style: {
            x: gridX - width / 2 + halfUnitWidth,
            y: chartStartY - timeScaleHeight - height / 2 + timeScaleHeight / 2
          }
        });
        // 5. 加到zrender实例中
        zr.add(dateText);
      }
      zr.add(gridLine);
    }

    // 1. 遍历tasks数组
    tasks.forEach(function (task, index) {
      // 2. 因为有最后一行是空行，没有任务，用来创建新任务，轮空不画
      if (!(task !== null && task !== void 0 && task.name)) return;
      // 3. 计算任务的绘制位置和矩形宽高
      var x = chartStartX + task.start * unitWidth;
      var y = chartStartY + (barHeight + barMargin) * index;
      var width = task.duration * unitWidth;
      var taskBarRect = {
        width: width,
        height: barHeight
      };
      // 4. 建一个组，设置可以拖拽 （感谢这个属性，后续交互省了很多力气，还可以设置只能垂直或者水平拖动）
      var group = new zrender.Group({
        x: x,
        y: y,
        draggable: true // Enable draggable for the group
        // draggable: "horizontal", // Enable draggable for the group
      });
      // 5. 创建任务跨度矩形
      var rect = new zrender.Rect({
        shape: {
          x: 0,
          y: 0,
          width: width,
          height: barHeight,
          r: 6
        },
        style: {
          fill: task.fillColor
        },
        cursor: 'move'
      });
      // 6. 加到组里
      group.add(rect);

      // Create a text shape for task name
      var taskName = new zrender.Text({
        style: {
          text: task.name,
          x: taskNamePaddingLeft,
          y: barHeight / 2 - 12 / 2,
          textFill: 'white',
          textAlign: 'left',
          textVerticalAlign: 'middle',
          fill: 'white'
        },
        cursor: 'move'
      });
      group.add(taskName);
      // Create a text shape for resource assignment
      var resourceText = new zrender.Text({
        style: {
          text: 'Assigned to: ' + task.resource,
          x: 0 + width + 5,
          y: barHeight / 2 + 0 - 12 / 2,
          textFill: 'black'
        },
        cursor: 'normal'
      });
      group.add(resourceText);
      var taskDurationText = new zrender.Text({
        style: {
          text: "".concat((0, _task.getRealDuration)(task, false), "\u5929"),
          x: width - taskNamePaddingLeft,
          y: barHeight / 2 - 12 / 2,
          textFill: 'white',
          textAlign: 'left',
          textVerticalAlign: 'middle',
          fill: 'white'
        },
        cursor: 'move'
      });
      var _taskDurationText$get = taskDurationText.getBoundingRect(),
        taskDurationTextWidth = _taskDurationText$get.width;
      taskDurationText.attr({
        style: {
          x: width - taskDurationTextWidth - taskNamePaddingLeft
        }
      });
      group.add(taskDurationText);
      zr.add(group);
    });
  };
  (0, _react.useEffect)(handleInitTimeLine, []);
  return /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
    style: {
      width: '100%',
      height: '100%',
      overflow: 'scroll'
    },
    children: /*#__PURE__*/(0, _jsxRuntime.jsx)("div", {
      id: "TimeLine",
      style: {
        width: 5000,
        height: 250,
        overflow: 'scroll'
      }
    })
  });
};
var _default = exports.default = TimeLine;
module.exports = exports.default;