"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useSearchParamsActions = exports.useSearchParams = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));
var _zustand = require("zustand");
var _enum = require("../type/enum");
var _utils = require("../utils");
var _storage = require("../utils/storage");
var useSearchParamsStore = (0, _zustand.create)(function (set, get) {
  var unitWidth = 160;
  var halfUnitWidth = 160 / 2;
  var barHeight = 30;
  var todayOffset = Math.floor((+new Date() - +new Date('2024-01-01')) / (60 * 60 * 24 * 1000));
  return {
    searchParams: (0, _storage.getItem)(_enum.StorageEnum.searchParams) || {
      currentGroup: null,
      debug: false,
      taskOwner: 'alexq',
      unitWidth: unitWidth,
      halfUnitWidth: halfUnitWidth,
      taskNamePaddingLeft: 15,
      initChartStartX: 1,
      initChartStartY: 50,
      timeScaleHeight: 20,
      milestoneTopHeight: 20,
      barHeight: 30,
      barMargin: 1,
      scrollSpeed: 35,
      includeHoliday: false,
      useLocal: false,
      useRemote: false,
      view: '',
      mockTaskSize: 0,
      todayOffset: todayOffset,
      initLastScrollX: (todayOffset - 1) * unitWidth,
      filter: undefined,
      showFilter: false,
      arrowSize: barHeight / 3 * 2,
      showArrow: true
    },
    actions: {
      refreshSearchParamsStore: function refreshSearchParamsStore() {
        var _params$barMargin, _params$view, _params$unitWidth;
        var params = (0, _utils.GetUrlParms)();
        var _get = get(),
          defaultSearchParams = _get[_enum.StorageEnum.searchParams];
        var newParams = (0, _objectSpread2.default)((0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultSearchParams), params), {}, {
          halfUnitWidth: ((params === null || params === void 0 ? void 0 : params.unitWidth) || unitWidth) / 2,
          barMargin: params !== null && params !== void 0 && params.debug ? 10 : (_params$barMargin = params === null || params === void 0 ? void 0 : params.barMargin) !== null && _params$barMargin !== void 0 ? _params$barMargin : 1,
          view: (_params$view = params === null || params === void 0 ? void 0 : params.view) !== null && _params$view !== void 0 ? _params$view : '',
          mockTaskSize: !(params !== null && params !== void 0 && params.useRemote) && !(params !== null && params !== void 0 && params.useLocal) && params !== null && params !== void 0 && params.mockTaskSize ? Number(params.mockTaskSize) : 0,
          initLastScrollX: ((params === null || params === void 0 ? void 0 : params.todayOffset) - 1) * ((_params$unitWidth = params === null || params === void 0 ? void 0 : params.unitWidth) !== null && _params$unitWidth !== void 0 ? _params$unitWidth : unitWidth)
        });
        set((0, _defineProperty2.default)({}, _enum.StorageEnum.searchParams, newParams));
        (0, _storage.setItem)(_enum.StorageEnum.searchParams, newParams);
      },
      setSearchParamsStore: function setSearchParamsStore(searchParams) {
        var _get2 = get(),
          defaultSearchParams = _get2[_enum.StorageEnum.searchParams];
        var newDefaultSearchParams = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultSearchParams), searchParams);
        set((0, _defineProperty2.default)({}, _enum.StorageEnum.searchParams, newDefaultSearchParams));
        (0, _storage.setItem)(_enum.StorageEnum.searchParams, newDefaultSearchParams);
      },
      clearSearchParamsStore: function clearSearchParamsStore() {
        return (0, _storage.removeItem)(_enum.StorageEnum.searchParams);
      }
    }
  };
});
var useSearchParams = exports.useSearchParams = function useSearchParams() {
  return useSearchParamsStore(function (state) {
    return state.searchParams;
  });
};
var useSearchParamsActions = exports.useSearchParamsActions = function useSearchParamsActions() {
  return useSearchParamsStore(function (state) {
    return state.actions;
  });
};